'' =================================================================================================
''
''   File....... jm_ping_demo.spin { not tested! }
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2017 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 03 APR 2017
''
'' =================================================================================================

{{

  Sonar Math:
 
  At sea level sound travels through air at 1130 feet per second. This equates to 1 inch in
  73.746us, or 1cm in 29.03us.
 
  Since the Ping sensor measures the time required for the sound wave to travel from the sensor and
  back, the result is divided by two when converting to microseconds -- this gives the distance to
  the target.

}}


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us
                                                                 
                        
con { io pins }                                                  
                                                                   
  RX1  = 31                                                     ' programming / terminal
  TX1  = 30                                                      
                                                                 
  SDA1 = 29                                                     ' eeprom / i2c
  SCL1 = 28                                                      
          
  PING =  0                                                     ' ping sensor (protect w/ 4.7K resistor)
                                                    
                                                                 
con                                                              
                                                                 
  #-1, ON, OFF                                                  ' digital control
                                                                 
                                                                 
obj                                                              
                                                                 
' main                                                          ' * master Spin cog
  time       : "jm_time_80"                                     '   timing and delays
  prng       : "jm_prng"                                        '   pseudo-random number generator
  io         : "jm_io_basic"                                    '   essential io
  term       : "jm_fullduplexserial"                            ' * serial IO for terminal
' background
                                                                 
' * uses cog when loaded                                         
                                                                 
                                                                 
var                                                              
                                                                 
                                                                 
pub main | t                                                        
                                                                 
  setup                                                          
                                                                 
  term.rxflush                                                  ' wait open terminal & keypress
  term.rx                                                        
  term.tx(term#CLS)                                              

  t := cnt
  repeat
    term.tx(term#HOME)
    
    term.str(string("PING us... "))
    term.dec(pingus)
    term.tx(term#CLREOL)
    term.tx(13)

    term.str(string("PING cm... "))
    term.dec(pingus * 100 / 29_03)
    term.tx(term#CLREOL)
    term.tx(13)

    term.str(string("PING in... "))
    term.dec(pingus * 1000 / 73_746 )
    term.tx(term#CLREOL)
    term.tx(13)

    waitcnt(t += constant(100 * MS_001))                        ' run loop every 100ms
                                                                 
                                                                 
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays
                                                                 
  prng.seed(-cnt, -cnt ~> 2, $EA7_BEEF, cnt <- 2, cnt)          ' seed randomizer
                                                                 
  io.start(0, 0)                                                ' clear all pins (master cog)
                                                                 
  term.start(RX1, TX1, %0000, 115_200)                          ' start serial for terminal

  bgcog := cognew(background, @bgstack) + 1                     ' start background cog
                                                                 

con

  { --------------- }
  {  S U P P O R T  }
  { --------------- }


var

  long  bgcog
  long  bgstack[32]

  long  millis


pri background | t

  millis := 0
  pingstate := 0

  ctra := %01000 << 26 | PING                                   ' set ctra for pos detect
  frqa := 1 

  ' background loop
  
  t := cnt
  repeat
    waitcnt(t += MS_001)
    ++millis
    scan_ping


var

  long  pingstate
  long  pingus


pri scan_ping                                                   ' scan/measure every 100ms                                                 '

  case pingstate
    0 :
      outa[PING] := 1                                           ' ping output high
      dira[PING] := 1
      waitcnt(cnt + constant(US_001 << 3))                      ' wait 8+ us
      dira[PING] := 0                                           ' set to input
      phsa := 0                                                 ' clear accumulator

    19 :
      pingus := phsa / constant(US_001 << 1)                    ' distance to target in us

    99 :
      pingstate := -1                                           ' reset                                         

  ++pingstate


dat { license }

  Copyright   byte      "Copyright (C) 2017 Jon McPhalen", 0                                                 

  
{{

  Copyright (C) 2017 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}